Module codec

Module codec 

Encoding and decoding utilities for Fiat-Shamir and group operations.

Structs

ByteSchnorrCodec
A byte-level Schnorr codec that works with any duplex sponge.

Traits

Codec
A trait defining the behavior of a domain-separated codec hashing, which is typically used for crate::traits::SigmaProtocols.

Functions

compute_iv
Compute the initialization vector (IV) for a protocol instance.

Type Aliases

KeccakByteSchnorrCodec
Type alias for a Keccak-based ByteSchnorrCodec. This is the codec used for matching test vectors from Sage.
Shake128DuplexSponge
Type alias for a SHAKE-based ByteSchnorrCodec.