pub struct ByteSchnorrCodec<G, H>where
G: PrimeGroup,
H: DuplexSpongeInterface,{ /* private fields */ }Expand description
A byte-level Schnorr codec that works with any duplex sponge.
This codec is generic over both the group G and the hash function H.
It can be used with different duplex sponge implementations.