Module composition

Module composition 

Protocol Composition with AND/OR Logic

This module defines the ComposedRelation enum, which generalizes the CanonicalLinearRelation by enabling compositional logic between multiple proof instances.

Specifically, it supports:

  • Simple atomic proofs (e.g., discrete logarithm, Pedersen commitments)
  • Conjunctions (And) of multiple sub-protocols
  • Disjunctions (Or) of multiple sub-protocols
  • Thresholds (Threshold) over multiple sub-protocols

Example Composition

And(
   Or(dleq, pedersen_commitment),
   Simple(discrete_logarithm),
   And(pedersen_commitment_dleq, bbs_blind_commitment_computation)
)

Structs

ComposedOrProverStateEntry
ComposedThresholdProverStateEntry

Enums

ComposedCommitment
ComposedProverState
ComposedRelation
A protocol proving knowledge of a witness for a composition of linear relations.
ComposedResponse
ComposedWitness

Type Aliases

ComposedOrProverState
ComposedThresholdProverState