ComposedRelation

Enum ComposedRelation 

pub enum ComposedRelation<G: PrimeGroup> {
    Simple(CanonicalLinearRelation<G>),
    And(Vec<ComposedRelation<G>>),
    Or(Vec<ComposedRelation<G>>),
    Threshold(usize, Vec<ComposedRelation<G>>),
}
A protocol proving knowledge of a witness for a composition of linear relations.

This implementation generalizes CanonicalLinearRelation by using AND/OR links.

Type Parameters

Variants

Implementations

Source

impl<G: PrimeGroup + ConstantTimeEq + ConditionallySelectable> ComposedRelation<G>

pub fn and<T: Into<ComposedRelation<G>>>( witness: impl IntoIterator<Item = T>, ) -> Self

Create a ComposedRelation for an AND relation from the given list of relations.

pub fn or<T: Into<ComposedRelation<G>>>( witness: impl IntoIterator<Item = T>, ) -> Self

Create a ComposedRelation for an OR relation from the given list of relations.

pub fn threshold<T: Into<ComposedRelation<G>>>( threshold: usize, witness: impl IntoIterator<Item = T>, ) -> Self

Create a ComposedRelation for a threshold relation from the given list of relations.

Source

impl<G> ComposedRelation<G>

pub fn into_nizk(self, session_identifier: &[u8]) -> Nizk<ComposedRelation<G>>

Convert this Protocol into a non-interactive zero-knowledge proof using the Shake128DuplexSponge codec and a specified session identifier.

This method provides a convenient way to create a NIZK from a Protocol without exposing the specific codec type to the API caller.

Parameters
  • session_identifier: Domain separator bytes for the Fiat-Shamir transform
Returns

A Nizk instance ready for proving and verification