pub type LinearCombination<G> = Sum<Weighted<Term<G>, <G as Group>::Scalar>>;Expand description
Represents a sparse linear combination of scalars and group elements.
For example, it can represent an equation like:
w_1 * (s_1 * P_1) + w_2 * (s_2 * P_2) + ... + w_n * (s_n * P_n)
where:
(s_i * P_i)are the terms, withs_iscalars (referenced byscalar_vars) andP_igroup elements (referenced byelement_vars).w_iare the constant weight scalars
The indices refer to external lists managed by the containing LinearMap.
Aliased Type
pub struct LinearCombination<G>(/* private fields */);