pub struct Nizk<P>{
pub session_id: Vec<u8>,
pub interactive_proof: P,
}Expand description
A Fiat-Shamir transformation of a SigmaProtocol into a non-interactive proof.
Nizk wraps an interactive Sigma protocol P
to produce non-interactive proofs by deriving verifier challenges from a
cryptographic sponge state.
Type Parameters
P: the Sigma protocol implementation.
Fields
session_id: Vec<u8>interactive_proof: PUnderlying interactive proof.
Implementations
Sourceimpl<P> Nizk<P>where
P: SigmaProtocol,
P::Challenge: PartialEq,
P::Commitment: NargSerialize + NargDeserialize + Encoding,
P::Response: NargSerialize + NargDeserialize + Encoding,
impl<P> Nizk<P>where
P: SigmaProtocol,
P::Challenge: PartialEq,
P::Commitment: NargSerialize + NargDeserialize + Encoding,
P::Response: NargSerialize + NargDeserialize + Encoding,
Sourcepub fn prove_batchable(
&self,
witness: &P::Witness,
rng: &mut impl ScalarRng,
) -> Result<Vec<u8>, Error>
pub fn prove_batchable( &self, witness: &P::Witness, rng: &mut impl ScalarRng, ) -> Result<Vec<u8>, Error>
Generates a batchable, serialized non-interactive proof.
Parameters
witness: The secret witness.rng: A cryptographically secure random number generator.
Returns
A serialized proof suitable for batch verification.
Panics
Panics if serialization fails (should not happen under correct implementation).
Examples found in repository?
More examples
examples/simple_composition.rs (line 60)
47fn prove(P1: G, x2: Scalar, H: G) -> ProofResult<Vec<u8>> {
48 // Compute public values
49 let P2 = G::generator() * x2;
50 let Q = H * x2;
51
52 let instance = create_relation(P1, P2, Q, H);
53 // Create OR witness with branch 1 being the real one (index 1)
54 let witness = ComposedWitness::Or(vec![
55 ComposedWitness::Simple(vec![Scalar::from(0u64)]),
56 ComposedWitness::Simple(vec![x2]),
57 ]);
58 let nizk = instance.into_nizk(b"or_proof_example");
59
60 nizk.prove_batchable(&witness, &mut OsRng)
61}Sourcepub fn verify_batchable(&self, narg_string: &[u8]) -> Result<(), Error>
pub fn verify_batchable(&self, narg_string: &[u8]) -> Result<(), Error>
Verifies a batchable non-interactive proof.
Parameters
proof: A serialized batchable proof.
Returns
Ok(())if the proof is valid.Err(Error)if deserialization or verification fails.
Errors
- Returns
Error::VerificationFailureif:- The challenge doesn’t match the recomputed one from the commitment.
- The response fails verification under the Sigma protocol.
Examples found in repository?
More examples
Sourceimpl<P> Nizk<P>where
P: SigmaProtocol + SigmaProtocolSimulator,
P::Challenge: PartialEq + NargDeserialize + NargSerialize,
impl<P> Nizk<P>where
P: SigmaProtocol + SigmaProtocolSimulator,
P::Challenge: PartialEq + NargDeserialize + NargSerialize,
Sourcepub fn prove_compact(
&self,
witness: &P::Witness,
rng: &mut impl ScalarRng,
) -> Result<Vec<u8>, Error>
pub fn prove_compact( &self, witness: &P::Witness, rng: &mut impl ScalarRng, ) -> Result<Vec<u8>, Error>
Sourcepub fn verify_compact(&self, proof: &[u8]) -> Result<(), Error>
pub fn verify_compact(&self, proof: &[u8]) -> Result<(), Error>
Verifies a compact proof.
Recomputes the commitment from the challenge and response, then verifies it.
Parameters
proof: A compact serialized proof.
Returns
Ok(())if the proof is valid.Err(Error)if deserialization or verification fails.
Errors
- Returns
Error::VerificationFailureif:- Deserialization fails.
- The recomputed commitment or response is invalid under the Sigma protocol.